#!/usr/bin/env python3
# coding=UTF-8
from functools import partial
from io import BytesIO
import logging
import math
from multiprocessing import Pool, cpu_count
import requests

import imagecolor

from .web_image import GoogleImageSearch
from .web_image import MicrosoftCognitiveImageSearch
from .exceptions import OversizeException, ZeroResultsException

"""Copyright © 2017 Rhys Hansen

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE."""

logger = logging.getLogger(__name__)


def average_image_url(url, name, timeout=5, max_size=5):
    """Takes an image url and averages the image.
    Arguments
    url: str
        url for image to average
    name: str
        name to use
    timeout: int
        request timeout in seconds
    max_size: int
        max size of request in MB
    return {red':r_avg, 'green':g_avg, 'blue':b_avg} or None
    """
    try:
        shorturl = url.rsplit('/', 1)[-1]
        logger.debug('Full URL,           %s', url)
        response = requests.get(url, timeout=timeout, stream=True)
        length = int(response.headers.get('Content-Length', 0))
        logger.debug('Request size,       %s - %.2fKB', shorturl, length/1024)
        if length > (1024 * 1024 * max_size):
            raise OversizeException(
                'Response size {2}MB, larger than {0}MB, discarding: {1}'
                .format(max_size, url, math.ceil(length/1024/1024)))
        logger.debug('Finished request,   %s', shorturl)
        result = imagecolor.average(BytesIO(response.content), name=name)
        logger.debug('Averaging complete, %s', shorturl)
        return(result)
    except OversizeException as e:
        logger.warning('Exception: %s', e)
        return(None)
    except Exception as e:
        logger.warning('Exception: %s @ %s', e, url)
        logger.debug('Traceback:', exc_info=True)
        return(None)


def _image_search_average(url_list, max_threads=2, **kwargs):
    """Takes a list of image urls and averages the images to get the
    average color. Designed to be implimented with many methods of
    url sourcing.
    Arguments
    url_list: list
        list of strings with the image urls to average
    max_threads: int
        max number of processes to spawn
    return {red':r_avg, 'green':g_avg, 'blue':b_avg} or None
    """
    if len(url_list) < 1:
        raise ZeroResultsException('No urls to average')
    r_total = 0
    g_total = 0
    b_total = 0
    imagecount = 0
    num_results = len(url_list)
    names = [n for n in range(num_results)]
    if num_results <= max_threads:
        threads = num_results
    else:
        threads = max_threads
    with Pool(threads) as p:
        results = p.starmap(partial(
            average_image_url, **kwargs), zip(url_list, names))
    logger.debug('All results averaged')
    for result in results:
        try:
            if result is not None:
                r_total += result['red']
                g_total += result['green']
                b_total += result['blue']
                imagecount += 1
        except TypeError:
            logger.debug('TypeError when iterating over results',
                         exc_info=True)
    logger.debug('Image count %d', imagecount)
    if imagecount > 0:
        logger.debug('Image count greater then 0')
        r_avg = int(r_total / imagecount)
        g_avg = int(g_total / imagecount)
        b_avg = int(b_total / imagecount)
        return({'red': r_avg, 'green': g_avg, 'blue': b_avg})
    else:
        raise ZeroResultsException('Nothing averaged successfully')


def google_average(search_term, num_results, api_key, cse_id, **kwargs):
    """Does a Google image search to get the average color of the
    top x results.
    Arguments
    search_term: str
        tearm to search for
    num_results: int
        number of results to average
    api_key: str
        Google API key
    cse_id: str
        Google CSE ID
    max_threads: int
        max number of processes to spawn

    return {'name':search_term, 'red':r_avg, 'green':g_avg, 'blue':b_avg}
    or None
    """
    url_list = []
    result = {'name': search_term}
    GIS = GoogleImageSearch(api_key, cse_id)
    url_list = GIS.search(search_term, num_results)
    result.update(_image_search_average(url_list, **kwargs))
    return(result)


def mscs_average(search_term, num_results, api_key, **kwargs):
    """Does a Microsoft Cognitive image search to get the average color of the
    top x results.
    Arguments
    search_term: str
        tearm to search for
    num_results: int
        number of results to average
    api_key: str
        Microsoft Cognitive API key
    max_threads: int
        max number of processes to spawn

    return {'name':search_term, 'red':r_avg, 'green':g_avg, 'blue':b_avg}
    or None
    """
    url_list = []
    result = {'name': search_term}
    MCIS = MicrosoftCognitiveImageSearch(api_key)
    url_list = MCIS.search(search_term, num_results)
    result.update(_image_search_average(url_list, **kwargs))
    return(result)



#!/usr/bin/env python3
# coding=UTF-8
import os
import sys
# installed
import pytest
# local
sys.path.append(os.path.split(os.path.split(__file__)[0])[0])
import searchcolor

google_api = os.environ["GOOGLE_SEARCH_API"]
cse = os.environ["GOOGLE_SEARCH_CSE"]


def test_api_keys():
    assert google_api is not None
    assert cse is not None


def test_google_average_red():
    result = searchcolor.google_average(
        'red', 10, google_api, cse, max_threads=8)
    assert result['name'] == 'red'
    assert result['red'] >= 0 and result['red'] <= 255
    assert result['green'] >= 0 and result['green'] <= 255
    assert result['blue'] >= 0 and result['blue'] <= 255
    assert result['red'] > result['green']
    assert result['red'] > result['blue']


def test_google_average_green():
    result = searchcolor.google_average(
        'green', 10, google_api, cse, max_threads=8)
    assert result['name'] == 'green'
    assert result['red'] >= 0 and result['red'] <= 255
    assert result['green'] >= 0 and result['green'] <= 255
    assert result['red'] >= 0 and result['blue'] <= 255
    assert result['green'] > result['red']
    assert result['green'] > result['blue']


def test_google_average_blue():
    result = searchcolor.google_average(
        'blue', 10, google_api, cse, max_threads=8)
    assert result['name'] == 'blue'
    assert result['red'] >= 0 and result['red'] <= 255
    assert result['green'] >= 0 and result['green'] <= 255
    assert result['blue'] >= 0 and result['blue'] <= 255
    assert result['blue'] > result['red']
    assert result['blue'] > result['green']



#!/usr/bin/env python3
# coding=UTF-8
from .average import average_image_url
from .average import google_average, mscs_average
from .exceptions import SearchColorException, OversizeException, ZeroResultsException

major_version = 2
minor_version = 1
build_version = 3

__author__ = 'Rhys Hansen'
__copyright__ = "Copyright 2017, Rhys Hansen"
__license__ = "MIT"
__version__ = '{0}.{1}.{2}'.format(major_version, minor_version, build_version)



#!/usr/bin/env python3
# coding=UTF-8
import logging
from googleapiclient.discovery import build
from py_ms_cognitive import PyMsCognitiveImageSearch
from .exceptions import ZeroResultsException

"""Copyright © 2017 Rhys Hansen

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE."""

logger = logging.getLogger(__name__)


class GoogleImageSearch(object):
    """Class for conducting Google Image Searches
    """
    def __init__(self, api_key, cse_id, file_type='jpg'):
        """Set up search object with your API key and CSE ID
        api_key: str
            Google API key
        cse_id: str
            Google CSE ID
        file_type: str
            file type restriction
        """
        self.api_key = api_key
        self.cse_id = cse_id
        self.file_type = file_type
        self.service = build("customsearch", "v1", developerKey=api_key)

    def search(self, search_term, num_results, **kwargs):
        """Gets x number of Google image result urls for
        a given search term.
        Arguments
        search_term: str
            tearm to search for
        num_results: int
            number of url results to return
        return ['url','url']
        """
        results = []
        count = 1
        try:
            while len(results) <= num_results:
                search_results = self.service.cse().list(
                    q=search_term,
                    cx=self.cse_id,
                    searchType="image",
                    fileType=self.file_type,
                    start=count,
                    **kwargs).execute()
                results.extend(
                    [r['link'] for r in search_results['items']])
                count += len(search_results)
            results = results[:num_results]
        except KeyError as e:
            logger.warning('Exception: %s', e)
        if len(results) == 0:
            raise ZeroResultsException("No images found")
        return(results)


class MicrosoftCognitiveImageSearch(object):
    """Class for conducting Microsoft Cognitive Image Searches."""
    def __init__(self, api_key):
        self.api_key = api_key

    def search(self, search_term, num_results, thumbnail=True, **kwargs):
        if num_results > 50:
            raise ValueError('Number of results requested greater than 50!')
        search_service = PyMsCognitiveImageSearch(
            self.api_key,
            search_term,
            **kwargs)  # custom_params="&safesearch=Off"
        search_results = search_service.search(
            limit=num_results,
            format='json')
        if thumbnail is True:
            return([r.thumbnail_url for r in search_results])
        else:
            return([r.content_url for r in search_results])



#!/usr/bin/env python3
# coding=UTF-8
from setuptools import setup
from codecs import open

major_version = 2
minor_version = 1
build_version = 3

version = '{0}.{1}.{2}'.format(major_version, minor_version, build_version)

setup(name='searchcolor',
      version=version,
      description='Image color extraction from web image search',
      long_description=open('README.rst', encoding='utf-8').read(),
      classifiers=[
          'Development Status :: 3 - Alpha',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.5',
          'Intended Audience :: Other Audience',
          'Topic :: Artistic Software',
          'Topic :: Multimedia :: Graphics',
          ],
      keywords='image average color search google',
      url='https://github.com/Tathorack/searchcolor',
      author='Rhys Hansen',
      author_email='rhyshonline@gmail.com',
      license='MIT',
      packages=['searchcolor'],
      install_requires=[
          'imagecolor>=1.0.0',
          'google-api-python-client',
          'requests',
          'py-ms-cognitive',
          ],
      setup_requires=[
          'pytest-runner',
          ],
      tests_require=[
          'pytest',
          ],
      include_package_data=True,
      zip_safe=False)
